/* ansi.c: (C) TOYODA Eizi 1994
 * NECK project
 * ESC sequences emulator routine --- not only ANSI
 */

#include <stdio.h>
#include "neck.h"

/* --- following sequences are of hterm --- */

/* CSI_Question():
 * emulation of kanji code setting of hterm.
 */
	static void
CSI_Question()
{
	int		c, p;
	p = 0;
	while (c = mygetb(MYFALSE), '0' <= c && c <= '9') {
		p *= 10;
		p += (c - '0');
	}
	if (c == 'i' && p == '4')
		myputs("\033A");					/* printer OFF */
	else if (c == 'i' && p == 5)
		myputs("\033@");					/* printer ON */
	else if (p == 25 && c == 'h')
		myputs("\033>LOCATE ,POS(0),1\r");	/* show cursor */
	else if (p == 25 && c == 'l')
		myputs("\033>LOCATE ,POS(0),0\r");	/* hide cursor */
	else if (p == 49 && c == 'h')
		msb_sjis(MYTRUE);						/* SJIS mode */
	else if (p == 49 && c == 'k')
		msb_sjis(MYFALSE);					/* EUC mode */
}

/* --- following sequences are quite popular --- */

/* CSI_MvCsr():
 * move cursor command
 */
	static void
CSI_MvCsr(c, n)
	int		c, n;
{
	int		p;
	switch (c) {
		case 'A': p = 0x1E; break;
		case 'B': p = 0x1F; break;
		case 'C': p = 0x1C; break;
		case 'D': p = 0x1D; break;
	}
	for ( ; n; n--)
		myputc(p);
}

/* CSI_Clear():
 * ^[[K or ^[[J command.
 */
	static void
CSI_Clear(c, param)
	int		c, param;
{
	if (c == 'K' && param == 0)
		myputc(0x05);
	else if (c == 'K' && param == 2)
		myputc(0x15);
	else if (c == 'J' && param == 0)
		myputc(ESC), myputc('T');
	else if (c == 'J' && param == 2)
		myputc(ESC), myputc('*');
	nulpadding(4);
}

/* CSI_XY():
 * gotoXY emulator.
 * NOTE: ESC [ pl ; pc H is 1 starting, and ESC = lc is 0 starting.
 */
	static void
CSI_YX(y, x)
	int		y, x;
{
	y += 0x1F; x += 0x1F;
	myputc(ESC); myputc('='); putbyte(x); putbyte(y);
	nulpadding(2);
}

/* csi_sav():
 * save/restore cursor position
 * CAUTION : not checked to work
 */
	void
csi_sav(c)
	int		c;
{
	if (c == 's') {
		myputs("\033>X=POS(0)\r");
		nulpadding(2);
		myputs("\033>Y=CSRLIN\r");
		nulpadding(2);
	} else if (c == 'u') {
		myputs("\033>LOCATE X,Y\r");
		nulpadding(2);
	}
}

/* CSI_Color():
 * Any color is expressed by blink attribute, so colors are
 * not distinct for user. Black makes characters invisible.
 * This routine is for MONOCHROME mode, not COLOR mode.
 * Unfortunately, COLOR 0 (default in MONOCHROME) means INVISIBLE
 * in COLOR. Default is, however, MONOCHROME and this program doesn't
 * change it, so this routine doesn't care COLOR mode.
 */
	static void
CSI_Color(cx, param)
	int		cx, *param;
{
	int		i, p;				/* p: parameter */
	int		d;					/* d: BASIC color switch */
	d = '0';
	for (i=0; i<=cx; i++) {
		p = param[i];
		if (p == 0)
			d = '0';
		else if (p == 4 || p == 5)
			d |= '4';
		else if (p == 7)
			d |= '3';
		else if (p == 40)
			d |= '2';
		else if (p == 30)
			d |= '1';
		else if (31 <= p && p <= 37)
			d |= '4';
		else if (41 <= p && p <= 47)
			d |= '6';
	}
	myputs("\033>COLOR ");
	myputc(d), myputc(CR);
	nulpadding(2);
}

/* ExecCSI():
 * execute a control sequences.
 */
	static void
ExecCSI(c, cx, param)
	int		c, cx, *param;
{
	if (c == 'A' || c == 'B' || c == 'C' || c == 'D') {
		/* take one param. default=1, 0 is nonsense */
		if (param[0] == 0)
			param[0] = 1;
		CSI_MvCsr(c, param[0]);
	} else if (c == 'H' || c == 'f') {
		/* take two params. default=0; */
		if (param[0] == 0)
			param[0] = 1;
		if (param[1] == 0)
			param[1] = 1;
		CSI_YX(param[0], param[1]);
	} else if (c == 'J' || c == 'K') {
		/* take a param. default=0 */
		CSI_Clear(c, param[0]);
	} else if (c == 's' || c == 'u') {
		/* take no param. */
		csi_sav(c);
	} else if (c == 'm') {
		/* take unlimited params. default=0 */
		CSI_Color(cx, param);
	} else {
		/* irresolvable command: display and alart */
		myputc(BEL);
		(void)myungetc(c);
	}
}

#define		N_PARAM		(16)

/* ansi_csi():
 * ESC [ entry point.
 */
	void
ansi_csi()
{
	int		c;
	int		cx;		/* number of parameters */
	int		param[N_PARAM];
	for (cx=0; cx<N_PARAM; cx++)
		param[cx] = 0;
	cx = 0;
	while ((c = mygetb(MYTRUE)) != EOF) {
		if ('0' <= c && c <= '9') {
			param[cx] *= 10;
			param[cx] += (c - '0');
		} else if (c == ';') {
			if (++cx == N_PARAM)
				break;
		} else if ('@' <= c && c < DEL) {
			ExecCSI(c, cx, param);
			break;
		} else if (c == '?') {
			CSI_Question();
			break;
		} else {
			(void)myungetc(c);
			break;
		}
	}
}

/* --- appendix -- neck original control sequence --- */

#define		OPTLEN		80

/*
 * ESC S: commandline option change
 *
 * (1) ESC S [aceksv279D] +
 *		turn on
 * (2) ESC S [aceksv279D] -
 *		turn off
 * (3) ESC S !
 *		initialize
 */
	void
escswitch()
{
	int		c;
	char	opt[OPTLEN], *p;
	p = opt;
	p[0] = NUL;
	while ((c = mygetb(MYTRUE)) != EOF) {
		if (c == '!') {
			initflag();
			break;
		} else if (c < SPC || DEL <= c) {
			break;
		} else if (c == '+') {
			for (p=opt; *p; p++)
				(void)optchar(*p, MYTRUE);
			break;
		} else if (c == '-') {
			for (p=opt; *p; p++)
				(void)optchar(*p, MYFALSE);
			break;
		} else {
			*p = c, p++, *p = NUL;
		}
	}
}
