/* io.c: (C) TOYODA Eizi 1994
 * for NECK Project
 * I/O handler
 */

#include	<stdio.h>
#ifdef	O_SJIS
#  include	<jctype.h>
#else
#  define	O_NECK 1
#endif
#include	"neck.h"

/* SI/SO control for 7bit env.ment */
#define		SI	(0x0f)
#define		SO	(0x0e)

static int	ungetc_buf = EOF;
static FILE	*InFile = NULL;
static boolean	IShiftedOut = MYFALSE;
static boolean	OShiftedOut = MYFALSE;
static boolean	ModeOutWchar = MYFALSE;

/************ input section *************/

/* myungetc():
 * cannot unget EOF, SI, SO.
 * return EOF means buffer overflow.
 */
	int
myungetc(c)
	int	c;
{
	if (ungetc_buf != EOF || c == EOF)
		return EOF;
	ungetc_buf = c;
	return c;
}

/* mygetc_s():
 * getchar equiv. with optionally errmsg and abortion.
 * SI/SO interpretation also supported.
 */
	int
mygetc_s(notabortflag, sisoflag)
	boolean notabortflag, sisoflag;
{
	int	c;
	if (ungetc_buf != EOF) {
		c = ungetc_buf;
		ungetc_buf = EOF;
	} else {
		c = fgetc(InFile);
	}
	if (c == EOF && !notabortflag) {
		eputs("neck: 1: unexpected EOF\n");
		quit(1);
	}
	/* recursive call */
	if (c == SI) {
		IShiftedOut = MYFALSE;
		c = mygetc(MYFALSE);
	}
	if (c == SO) {
		IShiftedOut = MYTRUE;
		c = mygetc(MYFALSE);
	}
	if (IShiftedOut && sisoflag && SPC < c) {
		c |= 0x80;
	}
	return c;
}

	boolean
myinfile(fn)
	char	*fn;
{
	if (InFile != NULL && InFile != stdin)
		(void)fclose((FILE *)InFile);
	if (streq(fn, "-")) {
		InFile = stdin;
	} else {
		/* "text" option may be troublesome on MS-DOS. */
		InFile = fopen(fn, "rt");
	}
	return (boolean)(InFile != NULL);
}

/************ output section *************/

/* PutByteOnly():
 * all terminal output calls finally this.
 */
	void
PutByteOnly(c)
	int		c;
{
	(void)fputc(c & 0xFF, stdout);
}

/* nulpadding():
 */
	void
nulpadding(n)
	int n;
{
#ifdef O_NECK
	while (n--)
		PutByteOnly(NUL);
#endif
	(void)fflush(stdout);
}

FILE	*logfile;

	void
openl()
{
	if (flags.D)
		logfile = fopen("neck.log", "w");
}

	void
closel()
{
	if (flags.D)
		(void)fclose(logfile);
}

	static void
LPutc(c)
	wc_t	c;
{
	if (!flags.D)
		return;
	if (c == SPC)
		c = '_';
	if (SPC < c && c < DEL)
		(void)fputc((int)c, logfile);
	else if (c <= M_DEL)
		(void)fprintf(logfile, "\n\\x%02X", (int)c);
	else
		(void)fprintf(logfile, "[%04X]", (int)c);
}

	static void
SingleCharMode()
{
	ModeOutWchar = MYFALSE;
#ifdef O_NECK
	PutByteOnly(ESC); PutByteOnly('H');
	nulpadding(4);
#endif
}

	static void
DoubleCharMode()
{
	ModeOutWchar = MYTRUE;
#ifdef O_NECK
	PutByteOnly(ESC); PutByteOnly('K');
	nulpadding(4);
#endif
}

	void
putbyte(c)
	int		c;
{
	PutByteOnly(c);
	LPutc((wc_t)c);
}

	void
myputs(s)
	char	*s;
{
	while (*s) {
		putbyte(*s);
		s++;
	}
}

	static	void
OSevenTurn(c)
	wc_t	c;
{
	if (OShiftedOut && !(c && 0x80)) {
		OShiftedOut = MYFALSE;
		putbyte(SI);
	}
	if (!OShiftedOut && (c && 0x80)) {
		OShiftedOut = MYTRUE;
		putbyte(SO);
	}
}

	static void
PutSingleChar(c)
	wc_t	c;
{
	if (ModeOutWchar)
		SingleCharMode();
	/*
	 * BACKQUOTE --- missing font in earlier 98
	 */
	if (c == 0x60 && !flags.v)
		c = 0xDE;
	/*
	 * BACKSLASH --- missing font in earlier 98
	 */
	if (c == BKSL98 && !flags.v)
		c = 0xEF;
	if (flags.oseven) {
		OSevenTurn(c);
		c &= 0x7F;
	}
	PutByteOnly((int)c);
	LPutc(c);
}

/* PutDoubleChar() */
	static void
PutDoubleChar(wc)
	wc_t	wc;
{
	if (!ModeOutWchar)
		DoubleCharMode();
#ifdef O_SJIS
	wc = jistojms(wc);
#endif /* O_SJIS */
	PutByteOnly((int)wc >> 8);
	PutByteOnly((int)wc & 0xFF);
	LPutc(wc);
}

/* myputwc():
 * output entry point
 */
	void
myputwc(hi, lo)
	int	hi, lo;		/* higer and lower bytes */
{
	wc_t	wc, *ws;
	wc = ((hi << 8) | (lo & 0xFF));
	ws = ksub(wc);
	while (*ws) {
		if (*ws >> 8 == 0) {
			PutSingleChar(*ws);
		} else {
			PutDoubleChar(*ws);
		}
		ws++;
	}
	(void)fflush(stdout);
};

/************* reset **************/

	void
initsiso()
{
	SingleCharMode();
	IShiftedOut = OShiftedOut = ModeOutWchar = MYFALSE;
}

