/* neck.c: (C) TOYODA Eizi 1994
 * main routines
 * SI/SO support is done in IO.C level.
 */

#include <stdio.h>
#include "neck.h"

#define	KANA_MAX	(0x5f)

static enum g0_t {
	CS_US_ASCII, CS_JIS_ROMAN, CS_98_ANK, CS_JIS_KANA,
	CS_JIS_78, CS_JIS_83
} G0Code = CS_98_ANK;

static enum g0_t LastRoman = CS_98_ANK;
static enum g0_t LastKanzi = CS_JIS_78;

	static int
SevenBitWcharMode()
{
	return (G0Code == CS_JIS_83 || G0Code == CS_JIS_78);
}

	static void
G0TurnTo(code)
	enum g0_t code;
{
	G0Code = code;
	if (SevenBitWcharMode()) {
		LastKanzi = code;
	} else if (code == CS_JIS_KANA) {
	} else if (flags.a) {
		LastRoman = G0Code = CS_US_ASCII;
	} else if (flags.ank) {
		LastRoman = G0Code = CS_98_ANK;
	} else {
		LastRoman = code;
	}
}

/*
 * ESC $ --- designation of multibyte character set
 * ESC $ @ --- G0 set to JIS78
 * ESC $ B --- G0 set to JIS83
 * ESC $ ( @ --- G0 set to JIS78
 * ESC $ ( B --- G0 set to JIS83
 * ESC $ ) @ --- G1 set to JIS78 --- practically equiv. to EUC
 * ESC $ ) B --- G1 set to JIS83 --- practically equiv. to EUC
 */
	static void
EscDol()
{
	int c;
	if ((c=mygetb(MYTRUE)) == EOF) return;
	if (c=='@') {
		G0TurnTo(CS_JIS_78);
	} else if (c == 'B') {
		G0TurnTo(CS_JIS_83);
	} else if (c == '(') {
		/* recursive call */
		EscDol();
	} else if (c == ')') {
		if ((c=mygetb(MYTRUE)) == EOF)
			return;
		if (c == '@' || c == 'B')
			msb_sjis(MYFALSE);
	} else {
		myputc('$'); putbyte(c);
	};
}

/* ESC ) --- G1 designation to singlebyte character set
 * ESC ) B --- G1 set to US-ASCII  (not yet)
 * ESC ) J --- G1 set to JIS-ROMAN (not yet)
 * ESC ) I --- G1 set to JIS-KANA (X 0201 KATAKANA)
 * --- practically same to SJIS.
 */
	static void
EscRen()
{
	int	c;
	if ((c=mygetb(MYTRUE)) == EOF)
		return;
	if (c == 'I')
		msb_sjis(MYTRUE);
}

/* ESC ( --- G0 designation to singlebyte character set
 * ESC ( B --- G0 set to US-ASCII
 * ESC ( H --- G0 set to JIS_ROMAN ?? (impl.98_ANK)
 * ESC ( I --- G0 set to JIS-KANA (X 0201 KATAKANA)
 * ESC ( J --- G0 set to JIS-ROMAN
 */
	static void
EscPar()
{
	int	c;
	if ((c=mygetb(MYTRUE)) == EOF) {
		/* VOID */
	} else if (c == 'J') {
		G0TurnTo(CS_JIS_ROMAN);
	} else if (c == 'H') {
		G0TurnTo(CS_98_ANK);
	} else if (c == 'I') {
		G0TurnTo(CS_JIS_KANA);
	} else if (c == 'B') {
		G0TurnTo(CS_US_ASCII);
	};
}

/* initially static, but now also called from escr.c */
	void 
rembasic(ch)
	int ch;
{
	int c;
	myputc(ESC), myputc(ch);
	while ((c = mygetb(MYFALSE)) != '\r') {
		putbyte(c);
	}
	myputc('\r');
	nulpadding(6);
}

	static void
EscEqual()
{
	int ca, cb;
	ca = mygetb(MYFALSE);
	if (ca < 0x20 || 0x20+79 < ca) {
		myputc(ca);
		return;
	}
	cb = mygetb(MYFALSE);
	if (cb < 0x20 || 0x20+25 < cb) {
		myputc(cb);
		return;
	}
	myputc(ESC), myputc('=');
	putbyte(ca), putbyte(cb);
	nulpadding(4);
	return;
}

	static void
EscPret()
{
	int c;
	if ((c=mygetb(MYTRUE)) == EOF) return;
	if (c=='$') {
		EscDol();
	} else if (c==')') {
		EscRen();
	} else if (c=='(') {
		EscPar();
	} else if (c=='[') {
		/* see ansi.c */
		ansi_csi();
	} else if (c=='.' || c=='>' || c=='<') {
		rembasic(c);
	} else if (c=='=') {
		(void)EscEqual();
	} else if (c=='c') {
		initsiso();
		G0TurnTo(LastRoman);
		myputs("\033*");
		nulpadding(8);
	} else if (c=='*' || c==' ') {
		initsiso();
		G0TurnTo(LastRoman);
		myputc(ESC), putbyte(c);
		nulpadding(8);
	} else if (c=='K') {
		G0TurnTo(LastKanzi);
	} else if (c=='H') {
		G0TurnTo(LastRoman);
	} else if (c=='S') {
		/* original neck control sequence (see ansi.c) */
		escswitch();
	} else if (c=='R') {
		/* emulate some hterm's sequence (see escr.c) */
		esc_r();
	} else if (c=='Y' || c=='T') {
		/*
		 * Y : clear to the end of scroll area
		 * T : clear all of the line, cursor goto column 0.
		 */
		myputc(ESC), putbyte(c);
		nulpadding(2);
	} else if (c=='#' || c=='\"' || c=='!' || c=='@' || c=='A') {
		/*
		 * # : disable key-in / \": enable key-in.
		 * ! : request ID code "NEC00000010\r".
		 * @ : LPT on / A : LPT off
		 */
		myputc(ESC), putbyte(c);
		nulpadding(2);
	} else if (c==ESC) {
		(void)myungetc(ESC);
	} else {
		myputc(ESC);
		nulpadding(2); /* separate ESC */
		myputc(c);
	};
}

	static void
SubPret()
{
	int		c;
	if ((c=mygetb(MYTRUE)) == EOF) return;
	if (c == 'p')
		G0TurnTo(LastKanzi);
	else if (c == 'q')
		G0TurnTo(LastRoman);
}

/* PutSevenWchar():
 * the second byte examination.
 */
	static void
PutSevenWchar(ca)
	int	ca;
{
	int	cb;
	if (ca == SPC) {
		myputc(SPC);
		return;
	}
	cb = mygetc(MYFALSE);
	if (cb <= SPC || DEL <= cb) {
		myputc(ca), myputc(cb);
		return;
	}
	myputwc(ca, cb);
}

/*
 */
	static void
PutC0Char(c)
	int	c;
{
	/*
	 * mode turn to LastRoman at newline: -n option.
	 * illegal but useful in case of ESC-sequence dropping.
	 */
	if (c == '\n' && flags.n && SevenBitWcharMode())
		G0Code = LastRoman;
	/*
	 * BackSpace erases character,
	 * and that is inappropriate for curses.
 	 */
	if (c == '\b')
		c = 0x1D;
	myputc(c);
}

/*
 */
	static void
PutSevenSgChar(c)
	int	c;
{
	if (c <= KANA_MAX && G0Code == CS_JIS_KANA)
		c |= 0x80;
	if (c == '\\' && G0Code == CS_US_ASCII)
		c = BKSL98;
	if (c == '~' && G0Code == CS_JIS_ROMAN)
		c = 0x94;
	myputc(c);
}

/* fileloop():
 * file loop
 * BUG: abort program at unexpected EOF and retcode 1.
 */
	void
fileloop(fn)
	char	*fn;
{
	int		c;
	if (!myinfile(fn)) {
		eputs(fn); eputs(": No such file.\n");
		return;
	}
	while (EOF != (c=mygetc(MYTRUE))) {
		if (c == ESC) {
			EscPret();
		} else if (c == SUB) {
			SubPret();
		} else if (c & 0x80) {
			/* ANY CHAR in RIGHT-128 */
			msb_on(c);
		} else if (c <= SPC || c == DEL) {
			/* CONTROL CHAR in LEFT-128 */
			PutC0Char(c);
		} else if (SevenBitWcharMode()) {
			PutSevenWchar(c);
		} else {
			PutSevenSgChar(c);
		};
	}
}
