/* necplot.c: TOYODA Eizi (GHE03056@niftyserve.or.jp)
 * for NECK project 1994.10.19
 * NEC PC-9801 ROM BASIC TERMINAL mode
 * unixplot compatibility library
 */

#include <stdio.h>
#include <math.h>
#include "neck.h"

/*
 * # defines
 */

#define PI			3.141592653589793
#define CHARWIDTH	8
#define	SXRANGE		400
#define	SXMAX		399

/*
 * static variables
 */

/* current graphical point */
static int			CpX = 0, CpY = 0;

/* linemod() result */
static char			*Plot_Linestyle = "&HFFFF";

/*
 * world coord. conversion.
 * set by space(), used by XCONV() and YCONV().
 */
static boolean	XRev = MYFALSE, YRev = MYTRUE;
static struct rectang_t {
	int			x0, y0, dx, dy;
} World;

#define XCONV(x)	((x) = ((XRev) ? -(x) : (x)))
#define YCONV(y)	((y) = ((YRev) ? -(y) : (y)))

/* sprintf() buffer */
#define SBUFLEN		256
static char			sbuf[SBUFLEN];

/*
 * internal function
 */

	static boolean
StrHead(s,r)
	char			*s, *r;
{
	while (*r) {
		if (0 == *s) return MYFALSE;
		if (*r != *s) return MYFALSE;
		r++, s++;
	}
	return MYTRUE;
}


/*
 * plot(3) compatibility functions
 */

linemod(s)
	char *s;
{
	char *ls;
	if (StrHead(s,"c.")) {
		(void)setcolor(s+2);
		return 1;
	}
	if (StrHead(s,"dotted"))
		ls= "&HAAAA";
	else if (StrHead(s,"shortdashed"))
		ls= "&HFCFC";
	else if (StrHead(s,"longdashed"))
		ls= "&HFFF0";
	else if (StrHead(s,"dotdashed"))
		ls= "&HFAFA";
	else
		ls= "&HFFFF";
	Plot_Linestyle = ls;
	return 0;
}

setcolor(s)
	char *s;
{
	char c;
	if (StrHead(s,"black"))
		c=0;
	else if (StrHead(s,"blue"))
		c=1;
	else if (StrHead(s,"red"))
		c=2;
	else if (StrHead(s,"purple") || StrHead(s,"magenta"))
		c=3;
	else if (StrHead(s,"green"))
		c=4;
	else if (StrHead(s,"cyan"))
		c=5;
	else if (StrHead(s,"yellow"))
		c=6;
	else
		c=7;
	myputs("\033.CONSOLE ,,,0\r");
	(void)sprintf(sbuf,"\033.COLOR ,,,%i\r",(int)c);
	myputs(sbuf);
	return 0;
}

openpl()
{
	/*
	 * this statement hides fkey mark
	 */
	myputs("\033.CONSOLE 0,24,0\r");
	/*
 	 * parameters for SCREEN statement:
	 * 1st: 0=COLOR-200, 1=BW-200, 2=BW-400, 3=COLOR-400(lines)
	 * 2nd: +1:hispeed flag, +2=killgraphics flag.
	 * 3rd: active page, 0 is most popular and safe.
	 * 4th: display page, 1 is most popular and safe.
	 */
#ifdef NIHYAKU_LINES
	/* this is for 200line COLOR display */
	myputs("\33>SCREEN 0,1,0,1\r");
	myputs("\33>WINDOW (-20,-20)-(419,419)\r");
	myputs("\33>VIEW (240,0)-(639,199)\r");
#else /* NIHYAKU_LINES */
	/* this is for 400line COLOR display */
	myputs("\33>SCREEN 3,1,0,1\r");
	myputs("\33>WINDOW (-20,-20)-(419,419)\r");
	myputs("\33>VIEW (240,0)-(639,399)\r");
#endif /* NIHYAKU_LINES */
	World.x0 = 0; World.dx = 400;
	return 0;
}

closepl()
{
	return 0;
}

erase() /* clear the window */
{
  myputs("\33>CLS 2\r");
  return 0;
}

space(x1,y1,x2,y2)
	int x1,y1,x2,y2;
{
	XRev = (boolean)(x1 > x2);
	YRev = (boolean)(y2 > y1);
	XCONV(x1); XCONV(x2);
	YCONV(y1); YCONV(y2);
	World.x0 = x1; World.dx = x2-x1;
	World.y0 = y1; World.dy = y2-y1;
	/* yohaku wo toru */
	x1 -= World.dx / 20;
	x2 += World.dx / 20;
	y1 -= World.dy / 20;
	y2 += World.dy / 20;
	(void)sprintf(sbuf, "\033>WINDOW (%i,%i)-(%i,%i)\r",
		x1, y2, x2-1, y1-1);
	myputs(sbuf);
	return 0;
}

/* argument must NOT be converted value! */
move(x,y)
	int x,y;
{
	CpX = x;  CpY = y;
	return 0;
}

point(x,y)
	int x,y;
{
	(void)move(x,y);
	XCONV(x); YCONV(y);
	(void)sprintf(sbuf, "\33>PSET (%i,%i)\r",x,y);
	myputs(sbuf);
	return 0;
}

cont(x,y)
int			x, y;
{
	int		xsav, ysav, oldx, oldy;
	xsav = x; ysav = y; oldx = CpX; oldy = CpY;
	XCONV(x); YCONV(y);
	XCONV(oldx); YCONV(oldy);
	(void)sprintf(sbuf, "\33.LINE (%i,%i)-(%i,%i),,, %s\r",
		oldx, oldy, x, y, Plot_Linestyle);
	myputs(sbuf);
	(void)move(xsav, ysav);
	nulpadding(20);
	return 0;
}

line(a,b,c,d)
	int a,b,c,d;
{
	(void)move(a,b);
	(void)cont(c,d);
	return 0;
}

rectangle(x1,y1,x2,y2)
	int x1,y1,x2,y2;
{
	(void)move(x2,y2);
	XCONV(x1); XCONV(x2);
	YCONV(y1); YCONV(y2);
	(void)sprintf(sbuf, "\33>LINE (%i,%i)-(%i,%i),,B\r",
		x1, y1, x2, y2);
	myputs(sbuf);
	return 0;
}

fillrectangle(x1,y1,x2,y2)
	int x1,y1,x2,y2;
{
	(void)move(x2,y2);
	XCONV(x1); XCONV(x2);
	YCONV(y1); YCONV(y2);
	(void)sprintf(sbuf, "\33>LINE (%i,%i)-(%i,%i),,BF\r",
		x1, y1, x2, y2);
	myputs(sbuf);
	return 0;
}

circle(x,y,r)
	int x,y,r;
{
	XCONV(x); YCONV(y);
	(void)sprintf(sbuf, "\33>CIRCLE (%i,%i),%i\r", x, y, r);
	myputs(sbuf);
	return 0;
}

label(s)
	char *s;
{
	int		x, y, Sdx;
	x = CpX, y = CpY;
	Sdx = 0;
	XCONV(x);
	YCONV(y);
	while (*s) {
		(void)sprintf(sbuf,
			"\033>PUT @(MAP(%i,0)+%i,MAP(%i,1)),KANJI(%i)\r",
			x, Sdx, y, (int)(unsigned char)*s);
		myputs(sbuf);
		nulpadding(20);
		Sdx += CHARWIDTH;
		if ((SXRANGE*((long)x-World.x0)/World.dx + Sdx) > SXMAX)
			break;
		s++;
	}
	return 0;
}

arc(x,y,x1,y1,x2,y2)
	int x,y,x1,x2,y1,y2;
{
	int		r;
	double	w1, w2;
	XCONV(x); XCONV(x1); XCONV(x2);
	YCONV(y); YCONV(y1); YCONV(y2);
	r = sqrt((double)((x-x1)*(x-x1)+(y-y1)*(y-y1)));
	w1 = atan2((double)(y1-y), (double)(x1-x));
	w2 = atan2((double)(y2-y), (double)(x2-x));
	if ((w2 -= w1) < 0.0)
		w2 += 2*PI;
	(void)sprintf(sbuf, "\33>CIRCLE (%i,%i),%i,,%3.0f,%3.0f\r",
		x, y, r, w1, w2);
	myputs(sbuf);
	return 0;
}
